clear;
close all;
clc;


InputFile='Borescope_Inputs_Design3.txt'; % Input file: individual inputs are described within the txt file


%% Read Inputs
TimeNow=clock;

fileID_r=fopen(InputFile,'r');
Text_Inputs=textscan(fileID_r,'%s', 'Delimiter', '\n');
Inputs=Text_Inputs{1};
for i=1:1:length(Inputs)
    eval(Inputs{i});
end
fclose(fileID_r);
OutputDirectory=strcat(FolderName,'_',sprintf('%d_%d_%d_%d_%d_%d',TimeNow(1,1),TimeNow(1,2),TimeNow(1,3),TimeNow(1,4),TimeNow(1,5),round(TimeNow(1,6))));
mkdir(OutputDirectory);
fileID_w=fopen(strcat(OutputDirectory,'\Inputs.txt'),'w');

for i=1:1:length(Inputs)
    fprintf(fileID_w,'%s', Inputs{i});
    fprintf(fileID_w,'\n\r');
    if isempty(Inputs{i})==0
        fprintf(fileID_w,'\n\r');
    end
end
fclose(fileID_w);

%% Allocation %%%%%%%%%%%%%%%%%%%%%%

tilt_angle=tilt_angle*pi/180; % beam tilt angle
dir_angle=dir_angle*pi/180; % beam directional angle for tilted beam
div_angle=div_angle*pi/180; % beam divergence angle
rot=rot*pi/180; % borescope rotation angle about z (only applicable if WithinEnclosure=1)

M=zeros(Nc*Np+1,8,14);
% Each line is a laser ray
% Columns are x, y, z, xd, yd, zd, OB, STOP
% Each layer is a new position

% x, y, z are coordinates at each control surface
% xd, yd, zd are coordinates of a dummy point that give the direction
% of the laser ray after the control surface

% OB is the optical behaviour of each ray at each control surface:
% 0 for refraction, 1 for reflection, 2 for ray parallel to optical
% surface, 3 means the laser path is no longer being calculated (failed at
% a previous control surface), 4 is out of lens path, 5 means stopped by blocker

% STOP is activated 1, when the calculation has found a ray not behaving as
% expected for a certain control surface. Calculation is stopped for this
% ray a this point. The result is displayed in red instead of blue

N=zeros(Nc*Np+1,3,7);
% Each line is for each laser ray
% Columns are x, y, z components of the vectors of the plane normal at the point of intersection
% Each layer is a new position

M0=zeros(Nc*Np+1,3,14);
% Each line is a laser ray
% Columns are x, y, z
% Each layer is a new position

% x, y, z are coordinates at each control surface

Colour_Gaussian=zeros(Nc*Np+1,4);
% Each line is a laser ray
% First column is Gaussian distribution of laser power
% 2 to 4 columns are RGB values

Ellipses=zeros(Nc*Np+1,3);
% Each line is a laser ray
% First column is a for each of the ellipses that compose the
% discretization of the beam
% Second column is b
% Third collum is angle position for each point


%% Enclosure Window

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Coordinates inside surface %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Coordinate rotation
q_Win_is=sqrt((Win_is_0(1,:)).*(Win_is_0(1,:))+(Win_is_0(2,:)).*(Win_is_0(2,:)));
angle_Win_is=atan2((Win_is_0(2,:)),(Win_is_0(1,:)))-rot;
Win_is_rot=[q_Win_is(1,1)*cos(angle_Win_is(1,1)) q_Win_is(1,2)*cos(angle_Win_is(1,2)) q_Win_is(1,1)*sin(angle_Win_is(1,1)) q_Win_is(1,2)*sin(angle_Win_is(1,2))];

% Allocation of all enclosure information into one variable
Win_is=[Win_is_rot(1,1) Win_is_rot(1,3) -h/2 Win_is_rot(1,2) Win_is_rot(1,4) -h/2 Win_is_rot(1,1) Win_is_rot(1,3) h/2 Win_is_rot(1,2) Win_is_rot(1,4) h/2];
Win_is_ABC=cross((Win_is(1,7:9)-Win_is(1,1:3)),(Win_is(1,4:6)-Win_is(1,1:3))); % Calculation of plane normal
Win_is(1,13:16)=[Win_is_ABC./norm(Win_is_ABC) -dot(Win_is_ABC./norm(Win_is_ABC),Win_is(1,1:3))]; % All information into a single variable


if Win_is(1,1)>=Win_is(1,4)
    Win_is_xmin=Win_is(1,4);
    Win_is_xmax=Win_is(1,1);
else
    Win_is_xmin=Win_is(1,1);
    Win_is_xmax=Win_is(1,4);
end

if Win_is(1,2)>=Win_is(1,5)
    Win_is_ymin=Win_is(1,5);
    Win_is_ymax=Win_is(1,2);
else
    Win_is_ymin=Win_is(1,2);
    Win_is_ymax=Win_is(1,5);
end

Win_is(1,17:20)=[Win_is_xmin Win_is_xmax Win_is_ymin Win_is_ymax];
% Win_is x1 y1 z1 x2 y2 z2 x3 y3 z3 x4 y4 z4 A B C D xmin xmax ymin ymax

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Coordinates outside surface %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


X_enclosure_0 = Enclosure_Coords(:,1)+origin(1);
Y_enclosure_0 = Enclosure_Coords(:,2)+origin(2);

% Coordinate rotation
q_enclosure=sqrt((X_enclosure_0).*(X_enclosure_0)+(Y_enclosure_0).*(Y_enclosure_0));
angle_enclosure=atan2((Y_enclosure_0),(X_enclosure_0))-rot;

X_enclosure=zeros(length(X_enclosure_0(1,:)),1);
Y_enclosure=zeros(length(X_enclosure_0(1,:)),1);
for v=1:1:length(X_enclosure_0)
    
    X_enclosure(v)=q_enclosure(v)*cos(angle_enclosure(v));
    Y_enclosure(v)=q_enclosure(v)*sin(angle_enclosure(v));
end

Win_os=zeros(length(X_enclosure(1,:))-1,20);

for v=1:1:length(X_enclosure)-1
    
    Win_os(v,1:12)=[X_enclosure(v) Y_enclosure(v) -h/2 X_enclosure(v+1) Y_enclosure(v+1) -h/2 X_enclosure(v) Y_enclosure(v) h/2 X_enclosure(v+1) Y_enclosure(v+1) h/2];
    Win_os_ABC=cross((Win_os(v,7:9)-Win_os(v,1:3)),(Win_os(v,4:6)-Win_os(v,1:3)));
    Win_os(v,13:16)=[Win_os_ABC./norm(Win_os_ABC) -dot(Win_os_ABC./norm(Win_os_ABC),Win_os(v,1:3))];
end

for v=1:1:length(X_enclosure)-1
    
    if Win_os(v,1)>=Win_os(v,4)
        Win_os_xmin=Win_os(v,4);
        Win_os_xmax=Win_os(v,1);
    else
        Win_os_xmin=Win_os(v,1);
        Win_os_xmax=Win_os(v,4);
    end
    
    if Win_os(v,2)>=Win_os(v,5)
        Win_os_ymin=Win_os(v,5);
        Win_os_ymax=Win_os(v,2);
    else
        Win_os_ymin=Win_os(v,2);
        Win_os_ymax=Win_os(v,5);
    end
    
    Win_os(v,17:20)=[Win_os_xmin Win_os_xmax Win_os_ymin Win_os_ymax];
    % Win_os x1 y1 z1 x2 y2 z2 x3 y3 z3 x4 y4 z4 A B C D xmin xmax ymin ymax
    
end


%% Laser starting point and Power Input
% Laser beam mesh (CS 1)

Rc=linspace(0,D/2,Nc+1); % Radius of each laser circle/ b parameter of the ellipse
a=ER*Rc;
thetac=linspace(0,2*pi,Np+1); % Angle position of each point on the circle

M(1,1:3,1)=[xd yd h]; % centre point of the beam
Colour_Gaussian(1,1)=gauss2mf(sqrt(M(1,1,1)^2+M(1,2,1)^2),[D*StDLaser/100 -D*MaxPowerLaser/2/100 D*StDLaser/100 D*MaxPowerLaser/2/100]);
k=2;
for i=2:1:Nc+1  % Beam meshing
    
    for j=1:1:Np
        if (((0<=thetac(j)) && (thetac(j)<=pi/2)) || ((3*pi/2<thetac(j)) && (thetac(j)<=2*pi)))
            M(k,1:3,1)=[xd+sqrt(1/((1/a(i)/a(i))+(tan(thetac(j))*tan(thetac(j))/Rc(i)/Rc(i)))) ...
                yd+(sqrt(1/((1/a(i)/a(i))+(tan(thetac(j))*tan(thetac(j))/Rc(i)/Rc(i)))))*tan(thetac(j)) h];
        else
            M(k,1:3,1)=[xd-sqrt(1/((1/a(i)/a(i))+(tan(thetac(j))*tan(thetac(j))/Rc(i)/Rc(i)))) ...
                yd-(sqrt(1/((1/a(i)/a(i))+(tan(thetac(j))*tan(thetac(j))/Rc(i)/Rc(i)))))*tan(thetac(j)) h];
        end
        
        Colour_Gaussian(k,1)=gauss2mf((a(i)*Rc(i))/sqrt(a(i)*a(i)*sin(thetac(j))*sin(thetac(j))+Rc(i)*Rc(i)*cos(thetac(j))*cos(thetac(j))),...
            [(a(end)*Rc(end))/sqrt(a(end)*a(end)*sin(thetac(j))*sin(thetac(j))+Rc(end)*Rc(end)*cos(thetac(j))*cos(thetac(j)))*2*StDLaser/100 ...
            -(a(end)*Rc(end))/sqrt(a(end)*a(end)*sin(thetac(j))*sin(thetac(j))+Rc(end)*Rc(end)*cos(thetac(j))*cos(thetac(j)))*MaxPowerLaser/100 ...
            (a(end)*Rc(end))/sqrt(a(end)*a(end)*sin(thetac(j))*sin(thetac(j))+Rc(end)*Rc(end)*cos(thetac(j))*cos(thetac(j)))*2*StDLaser/100 ...
            (a(end)*Rc(end))/sqrt(a(end)*a(end)*sin(thetac(j))*sin(thetac(j))+Rc(end)*Rc(end)*cos(thetac(j))*cos(thetac(j)))*MaxPowerLaser/100]);
        
        Ellipses(k,:)=[a(i) Rc(i) thetac(j)];
        
        
        k=k+1;
    end
end


if strcmp(ColourPlot,'jet')==1 % Colour beam options
    
    cmap=jet(1024);
    cmap(:,4)=linspace(0,1,1024);
    
    for k=1:1:length(M(:,1,1))
        c=1;
        while cmap(c,4)<Colour_Gaussian(k,1)
            c=c+1;
        end
        
        if c==1 || c==length(cmap)
            RC=cmap(c,1);
            GC=cmap(c,2);
            BC=cmap(c,3);
        else
            RC=mean([cmap(c,1) cmap(c-1,1)]);
            GC=mean([cmap(c,2) cmap(c-1,2)]);
            BC=mean([cmap(c,3) cmap(c-1,3)]);
        end
        Colour_Gaussian(k,2:4)=[RC GC BC];
        
    end
    
end

if strcmp(ColourPlot,'yellow')==1
    for k=1:1:length(M(:,1,1))
        Colour_Gaussian(k,2:4)=[1 1 0];
    end
end

if strcmp(ColourPlot,'magenta')==1
    for k=1:1:length(M(:,1,1))
        Colour_Gaussian(k,2:4)=[1 0 1];
    end
end

if strcmp(ColourPlot,'cyan')==1
    for k=1:1:length(M(:,1,1))
        Colour_Gaussian(k,2:4)=[0 1 1];
    end
end

if strcmp(ColourPlot,'red')==1
    for k=1:1:length(M(:,1,1))
        Colour_Gaussian(k,2:4)=[1 0 0];
    end
end

if strcmp(ColourPlot,'green')==1
    for k=1:1:length(M(:,1,1))
        Colour_Gaussian(k,2:4)=[0 1 0];
    end
end

if strcmp(ColourPlot,'blue')==1
    for k=1:1:length(M(:,1,1))
        Colour_Gaussian(k,2:4)=[0 0 1];
    end
end

if strcmp(ColourPlot,'white')==1
    for k=1:1:length(M(:,1,1))
        Colour_Gaussian(k,2:4)=[1 1 1];
    end
end

if strcmp(ColourPlot,'black')==1
    for k=1:1:length(M(:,1,1))
        Colour_Gaussian(k,2:4)=[0 0 0];
    end
end



for k=1:1:length(M(:,1,1))
    
    
    M(k,4:6,1)=[M(k,1,1)+sin(div_angle*Ellipses(k,1)/Ellipses(end,1)*cos(atan2(M(k,2,1),M(k,1,1)))+tilt_angle*cos(dir_angle))...
        M(k,2,1)+sin(div_angle*Ellipses(k,1)/Ellipses(end,1)*sin(atan2(M(k,2,1),M(k,1,1)))+tilt_angle*sin(dir_angle))...
        M(k,3,1)-cos(div_angle*Ellipses(k,1)/Ellipses(end,1)+tilt_angle)]; % dummy points to create a direction vector of the beam
    
    
    % Compare laser diameter to useful area of first lens encountered
    if lz_N==2
        if sqrt(M(k,1,1)^2+M(k,2,1)^2)>lz2_dmax/2
            M(k,7:8,1)=[4 1]; % out of the area for the first lens encountered
        end
    end
    
    if lz_N==1
        if sqrt(M(k,1,1)^2+M(k,2,1)^2)>lz1_dmax/2
            M(k,7:8,1)=[4 1]; % out of the area for the first lens encountered
        end
    end
    
    if lz_N==0
        if sqrt(M(k,1,1)^2+M(k,2,1)^2)>lR_e_dmax/2
            M(k,7:8,1)=[4 1]; % out of the area for the first lens encountered
        end
    end
    
    
    
    if M(k,1,1)>Blocker_x_pos
        M(k,7:8,1)=[5 1]; % cut out by blocker
    end
    if M(k,1,1)<Blocker_x_neg
        M(k,7:8,1)=[5 1]; % cut out by blocker
    end
    if M(k,2,1)>Blocker_y_pos
        M(k,7:8,1)=[5 1]; % cut out by blocker
    end
    if M(k,2,1)>Blocker_y_pos
        M(k,7:8,1)=[5 1]; % cut out by blocker
    end
    
    
end

%% Lens z2

if lz_N==2
    if Modular==1
        if lz2_Plano==0
            % Laser on the curved surface of the second lens (lz2) on top of the reflective lens (CS 2)
            lz2_c_coord=[0, 0, zg2+lz2_ct/2+((-1)^(lz2_cc))*lz2_R]; % Coordinates of the centre point of the geometry
            [M, N]=LensCurvedSurface_z(2, M, N, 0, lz2_R, lz2_c_coord, lz2_type, lz2_cc, lz2_dmax, lz2_n, 'Inlet', 0, 0);
            
            % Laser on the curved surface of the second lens (lz2) on top of the reflective lens (CS 3)
            lz2_c_coord=[0, 0, zg2-lz2_ct/2-((-1)^(lz2_cc))*lz2_R]; % Coordinates of the centre point of the geometry
            [M, N]=LensCurvedSurface_z(3, M, N, 1, lz2_R, lz2_c_coord, lz2_type, lz2_cc, lz2_dmax, lz2_n, 'Outlet', 0, 0);
            
            
        else if lz2_Plano==1
                
                if lz2_Inv==0
                    % Laser on the curved surface of the second lens (lz2) on top of the reflective lens (CS 2)
                    lz2_c_coord=[0, 0, zg2+((-1)^(lz2_Inv))*lz2_ct/2+((-1)^(lz2_cc+lz2_Inv))*lz2_R]; % Coordinates of the centre point of the geometry
                    [M, N]=LensCurvedSurface_z(2, M, N, lz2_Inv, lz2_R, lz2_c_coord, lz2_type, lz2_cc, lz2_dmax, lz2_n, 'Inlet', 0, 0);
                    
                    % Laser on the plane surface of the second lens (lz2) on top of the reflective lens (CS 3)
                    lz2_b_coord=[0, 0, zg2-((-1)^(lz2_Inv))*lz2_ct/2]; % Coordinates of the centre point of the geometry
                    [M, N]=LensPlaneSurface_z(3, M, N, lz2_b_coord, lz2_dmax, lz2_n, 'Outlet', 0, 0);
                    
                else if lz2_Inv==1
                        
                        % Laser on the plane surface of the second lens (lz2) on top of the reflective lens (CS 2)
                        lz2_b_coord=[0, 0, zg2-((-1)^(lz2_Inv))*lz2_ct/2]; % Coordinates of the centre point of the geometry
                        [M, N]=LensPlaneSurface_z(2, M, N, lz2_b_coord, lz2_dmax, lz2_n, 'Inlet', 0, 0);
                        
                        % Laser on the curved surface of the second lens (lz2) on top of the reflective lens (CS 3)
                        lz2_c_coord=[0, 0, zg2+((-1)^(lz2_Inv))*lz2_ct/2+((-1)^(lz2_cc+lz2_Inv))*lz2_R]; % Coordinates of the centre point of the geometry
                        [M, N]=LensCurvedSurface_z(3, M, N, lz2_Inv, lz2_R, lz2_c_coord, lz2_type, lz2_cc, lz2_dmax, lz2_n, 'Outlet', 0, 0);
                        
                    end
                end
                
                
            else if lz2_Plano==2
                    
                    % Laser on the plane surface of the second lens (lz2) on top of the reflective lens (CS 2)
                    lz2_b_coord=[0, 0, zg2+lz2_ct/2]; % Coordinates of the centre point of the geometry
                    [M, N]=LensPlaneSurface_z(2, M, N, lz2_b_coord, lz2_dmax, lz2_n, 'Inlet', 0, 0);
                    
                    % Laser on the plane surface of the second lens (lz2) on top of the reflective lens (CS 3)
                    lz2_b_coord=[0, 0, zg2-lz2_ct/2]; % Coordinates of the centre point of the geometry
                    [M, N]=LensPlaneSurface_z(3, M, N, lz2_b_coord, lz2_dmax, lz2_n, 'Outlet', 0, 0);
                end
            end
        end
    else % Not applicable to an integrated design borescope
        
        M(:,:,2)=M(:,:,1);
        M(:,:,3)=M(:,:,2);
        
    end
else
    M(:,:,2)=M(:,:,1); % Second lens z is ignored if number of lens on top is smaller than 2
    M(:,:,3)=M(:,:,2); % Second lens z is ignored if number of lens on top is smaller than 2
end





%% Lens z1

if lz_N>0
    if Modular==1
        if lz1_Plano==0
            % Laser on the curved surface of the first lens (lz1) on top of the reflective lens (CS 4)
            lz1_c_coord=[0, 0, zg1+lz1_ct/2+((-1)^(lz1_cc))*lz1_R]; % Coordinates of the centre point of the geometry
            [M, N]=LensCurvedSurface_z(4, M, N, 0, lz1_R, lz1_c_coord, lz1_type, lz1_cc, lz1_dmax, lz1_n, 'Inlet', 0, 0);
            
            % Laser on the curved surface of the first lens (lz1) on top of the reflective lens (CS 5)
            lz1_c_coord=[0, 0, zg1-lz1_ct/2-((-1)^(lz1_cc))*lz1_R]; % Coordinates of the centre point of the geometry
            [M, N]=LensCurvedSurface_z(5, M, N, 1, lz1_R, lz1_c_coord, lz1_type, lz1_cc, lz1_dmax, lz1_n, 'Outlet', 0, 0);
            
        else if lz1_Plano==1
                
                if lz1_Inv==0
                    
                    % Laser on the curved surface of the first lens (lz1) on top of the reflective lens (CS 4)
                    lz1_c_coord=[0, 0, zg1+((-1)^(lz1_Inv))*lz1_ct/2+((-1)^(lz1_cc+lz1_Inv))*lz1_R]; % Coordinates of the centre point of the geometry
                    [M, N]=LensCurvedSurface_z(4, M, N, lz1_Inv, lz1_R, lz1_c_coord, lz1_type, lz1_cc, lz1_dmax, lz1_n, 'Inlet', 0, 0);
                    
                    % Laser on the plane surface of the first lens (lz1) on top of the reflective lens (CS 5)
                    lz1_b_coord=[0, 0, zg1-((-1)^(lz1_Inv))*lz1_ct/2]; % Coordinates of the centre point of the geometry
                    [M, N]=LensPlaneSurface_z(5, M, N, lz1_b_coord, lz1_dmax, lz1_n, 'Outlet', 0, 0);
                    
                else if lz1_Inv==1
                        
                        % Laser on the plane surface of the first lens (lz1) on top of the reflective lens (CS 4)
                        lz1_b_coord=[0, 0, zg1-((-1)^(lz1_Inv))*lz1_ct/2]; % Coordinates of the centre point of the geometry
                        [M, N]=LensPlaneSurface_z(4, M, N, lz1_b_coord, lz1_dmax, lz1_n, 'Inlet', 0, 0);
                        
                        % Laser on the curved surface of the first lens (lz1) on top of the reflective lens (CS 5)
                        lz1_c_coord=[0, 0, zg1+((-1)^(lz1_Inv))*lz1_ct/2+((-1)^(lz1_cc+lz1_Inv))*lz1_R]; % Coordinates of the centre point of the geometry
                        [M, N]=LensCurvedSurface_z(5, M, N, lz1_Inv, lz1_R, lz1_c_coord, lz1_type, lz1_cc, lz1_dmax, lz1_n, 'Outlet', 0, 0);
                        
                    end
                end
                
            else if lz1_Plano==2
                    % Laser on the plane surface of the first lens (lz1) on top of the reflective lens (CS 4)
                    lz1_b_coord=[0, 0, zg1+lz1_ct/2]; % Coordinates of the centre point of the geometry
                    [M, N]=LensPlaneSurface_z(4, M, N, lz1_b_coord, lz1_dmax, lz1_n, 'Inlet', 0, 0);
                    
                    % Laser on the plane surface of the first lens (lz1) on top of the reflective lens (CS 5)
                    lz1_b_coord=[0, 0, zg1-lz1_ct/2]; % Coordinates of the centre point of the geometry
                    [M, N]=LensPlaneSurface_z(5, M, N, lz1_b_coord, lz1_dmax, lz1_n, 'Outlet', 0, 0);
                    
                end
            end
        end
        
    else % Only inlet on an integrated design borescope
        
        if lz1_Plano==2 % Flat surface inlet
            
            % Laser on the plane surface of the first lens (lz1) on top of the reflective lens (CS 4)
            lz1_b_coord=[0, 0, zg1+lz1_ct/2]; % Coordinates of the centre point of the geometry
            [M, N]=LensPlaneSurface_z(4, M, N, lz1_b_coord, lz1_dmax, lz1_n, 'Inlet', 0, 0);
            
        else
            
            % Laser on the curved surface of the first lens (lz1) on top of the reflective lens (CS 4)
            lz1_c_coord=[0, 0, zg1+lz1_ct/2+((-1)^(lz1_cc))*lz1_R]; % Coordinates of the centre point of the lens geometry
            [M, N]=LensCurvedSurface_z(4, M, N, 0, lz1_R, lz1_c_coord, lz1_type, lz1_cc, lz1_dmax, lz1_n, 'Inlet', 0, 0);
            
            
        end
        
        M(:,:,5)=M(:,:,4);
        
    end
    
else
    M(:,:,4)=M(:,:,3); % First lens z is ignored if number of lens on top is smaller than 1
    M(:,:,5)=M(:,:,4); % First lens z is ignored if number of lens on top is smaller than 1
end


%% Laser on the reflective lens

if Modular==1 && lR_PM==0
    
    % Laser on the entry surface of the reflective lens (lR) (CS 6)
    [M, N]=LensPlaneSurface_z_Prism(6, M, N, lR_l, lR_PA, lR_e_dmax, lR_n, 'Inlet', 0, 0);
    
else % Not applicable to an integrated design borescope
    M(:,:,6)=M(:,:,5);
    
end

% Laser on the reflective surface of the reflective lens (lR) (CS 7)
for k=1:1:length(M(:,1,1))
    
    if M(k,8,6)==1
        M(k,1:6,7)=M(k,1:6,6); % Second lens z is ignored if number of lens on top is smaller than 2
        M(k,7:8,7)=[3 M(k,8,6)]; % Laser ray failed before
        
    else
        % Laser ray vector
        a7=M(k,4,6)-M(k,1,6);
        b7=M(k,5,6)-M(k,2,6);
        c7=M(k,6,6)-M(k,3,6);
        
        if lR_type==0 % plane at lR_PA degrees
            
            % Coordinates of the centre point of the geometry
            lR_R_coord=[0, 0, lR_l/2];
            
            % Plane Coefficients
            theta7=0;
            phi7=lR_PA*pi/180;
            A7=sin(phi7)*cos(theta7);
            B7=sin(phi7)*sin(theta7);
            C7=cos(phi7);
            D7=0;
            
            %%% Intersection of the laser ray with the geometry
            % Intersection equations
            M(k,3,7)=(A7*a7/c7*M(k,3,6)-A7*M(k,1,6)+B7*b7/c7*M(k,3,6)-B7*M(k,2,6)-D7)/(A7*a7/c7+B7*b7/c7+C7);
            M(k,2,7)=b7/c7*(M(k,3,7)-M(k,3,6))+M(k,2,6);
            M(k,1,7)=a7/c7*(M(k,3,7)-M(k,3,6))+M(k,1,6);
            
            N(k,1:3,7)=[A7 B7 C7];
            
        else if lR_type==1 % sphere cut
                
                % Coordinates of the centre point of the geometry
                lR_R_coord=[-lR_R*cos(pi/4), 0, -lR_R*sin(pi/4)];
                
                % Quadratic formula a,b,c coefficients
                ai=(a7/c7)^2+(b7/c7)^2+1;
                bi=2*a7/c7*(M(k,1,6)-lR_R_coord(1,1))-2*lR_R_coord(1,3)-2*M(k,3,6)*(a7/c7)^2+...
                    2*b7/c7*(M(k,2,6)-lR_R_coord(1,2))-2*M(k,3,6)*(b7/c7)^2;
                ci=M(k,3,6)^2*(a7/c7)^2+(M(k,1,6)-lR_R_coord(1,1))^2+lR_R_coord(1,3)^2-lR_R^2-2*a7/c7*(M(k,1,3)-lR_R_coord(1,1))*M(k,3,6)+...
                    M(k,3,6)^2*(b7/c7)^2+(M(k,2,6)-lR_R_coord(1,2))^2-2*b7/c7*(M(k,2,6)-lR_R_coord(1,2))*M(k,3,6);
                
                % Intersection equations
                M(k,3,7)=(-bi+sqrt(bi^2-4*ai*ci))/(2*ai);
                M(k,2,7)=b7/c7*(M(k,3,7)-M(k,3,6))+M(k,2,6);
                M(k,1,7)=a7/c7*(M(k,3,7)-M(k,3,6))+M(k,1,6);
                
                N(k,1:3,7)=[M(k,1,7)-lR_R_coord(1,1) M(k,2,7)-lR_R_coord(1,2) M(k,3,7)-lR_R_coord(1,3)];
                
            else if lR_type==2 % cylinder cut
                    
                    
                    % Coordinates of the centre point of the geometry
                    lR_R_coord=[-lR_RC/2, 0, lR_HC/2];
                    
                    % Quadratic formula a,b,c coefficients
                    ai=(a7/c7)^2+(b7/c7)^2-(lR_RC/lR_HC)^2;
                    bi=2*a7/c7*(M(k,1,6)-lR_R_coord(1,1))+2*lR_R_coord(1,3)*(lR_RC/lR_HC)^2-2*M(k,3,6)*(a7/c7)^2+...
                        2*b7/c7*(M(k,2,6)-lR_R_coord(1,2))-2*M(k,3,6)*(b7/c7)^2;
                    ci=M(k,3,6)^2*(a7/c7)^2+(M(k,1,6)-lR_R_coord(1,1))^2-2*a7/c7*(M(k,1,6)-lR_R_coord(1,1))*M(k,3,6)+...
                        M(k,3,6)^2*(b7/c7)^2+(M(k,2,6)-lR_R_coord(1,2))^2-2*b7/c7*(M(k,2,6)-lR_R_coord(1,2))*M(k,3,6)-lR_R_coord(1,3)^2*(lR_RC/lR_HC)^2;
                    
                    % Intersection equations
                    M(k,3,7)=(-bi+sqrt(bi^2-4*ai*ci))/(2*ai);
                    M(k,2,7)=b7/c7*(M(k,3,7)-M(k,3,6))+M(k,2,6);
                    M(k,1,7)=a7/c7*(M(k,3,7)-M(k,3,6))+M(k,1,6);
                    
                    % (Different for the cone cut)
                    theta7=atan2(M(k,2,7),M(k,1,7)-lR_R_coord(1,1));
                    phi7=atan(lR_HC/lR_RC);
                    A7=sin(phi7)*cos(theta7);
                    B7=sin(phi7)*sin(theta7);
                    C7=cos(phi7);
                    D7=-A7*M(k,1,7)-B7*M(k,2,7)-C7*M(k,3,7);
                    
                    N(k,1:3,7)=[A7 B7 C7];
                    
                end
            end
        end
        
        
        if lR_PM==1 % Just a reflective surface
            [M(k,4:6,7), M(k,7,7), M(k,8,7)]=Optics(M(k,1:3,6), M(k,1:3,7), N(k,1:3,7), 1/lR_n, lR_FR, 1);
        else
            [M(k,4:6,7), M(k,7,7), M(k,8,7)]=Optics(M(k,1:3,6), M(k,1:3,7), N(k,1:3,7), lR_n, lR_FR, 1);
        end
        
    end
end


% Laser on the outlet surface of the reflective lens (lR) (CS 8)
if Modular==1 && lR_PM==0
    [M, N]=LensPlaneSurface_x_Prism(8, M, N, lR_l, lR_PA, lR_o_dmax, lR_n, 'Outlet', 0, 0);
    
else % Not applicable to an integrated design borescope
    M(:,:,8)=M(:,:,7);
    
end

%% lx1

if lx_N>=1
    
    if Modular==1
        if lx1_Plano==0
            
            % Laser on the curved surface of the first lens (lx1) in front of the reflective lens (CS 9)
            lx1_c_coord=[xg1-lx1_ct/2-((-1)^(lx1_cc))*lx1_R, 0, 0]; % Coordinates of the centre point of the geometry
            [M, N]=LensCurvedSurface_x(9, M, N, 1, lx1_R, lx1_c_coord, lx1_type, lx1_cc, lx1_dmax, lx1_n, 'Inlet', 0, 0);
            
            % Laser on the curved surface of the first lens (lx1) in front of the reflective lens (CS 10)
            lx1_c_coord=[xg1+lx1_ct/2+((-1)^(lx1_cc))*lx1_R, 0, 0]; % Coordinates of the centre point of the geometry
            [M, N]=LensCurvedSurface_x(10, M, N, 0, lx1_R, lx1_c_coord, lx1_type, lx1_cc, lx1_dmax, lx1_n, 'Outlet', 0, 0);
            
            
            
            
        else if lx1_Plano==1
                
                if lx1_Inv==0
                    
                    % Laser on the plane surface of the first lens (lx1) in front of the reflective lens (CS 9)
                    lx1_b_coord=[xg1-((-1)^(lx1_Inv))*lx1_ct/2, 0, 0]; % Coordinates of the centre point of the geometry
                    [M, N]=LensPlaneSurface_x(9, M, N, lx1_b_coord, lx1_dmax, lx1_n, 'Inlet', 0, 0);
                    
                    % Laser on the curved surface of the first lens (lx1) in front of the reflective lens (CS 10)
                    lx1_c_coord=[xg1+((-1)^(lx1_Inv))*lx1_ct/2+((-1)^(lx1_cc+lx1_Inv))*lx1_R, 0, 0]; % Coordinates of the centre point of the geometry
                    [M, N]=LensCurvedSurface_x(10, M, N, lx1_Inv, lx1_R, lx1_c_coord, lx1_type, lx1_cc, lx1_dmax, lx1_n, 'Outlet', 0, 0);
                    
                else if lx1_Inv==1
                        
                        % Laser on the curved surface of the first lens (lx1) in front of the reflective lens (CS 9)
                        lx1_c_coord=[xg1+((-1)^(lx1_Inv))*lx1_ct/2+((-1)^(lx1_cc+lx1_Inv))*lx1_R, 0, 0]; % Coordinates of the centre point of the geometry
                        [M, N]=LensCurvedSurface_x(9, M, N, lx1_Inv, lx1_R, lx1_c_coord, lx1_type, lx1_cc, lx1_dmax, lx1_n, 'Inlet', 0, 0);
                        
                        % Laser on the plane surface of the first lens (lx1) in front of the reflective lens (CS 10)
                        lx1_b_coord=[xg1-((-1)^(lx1_Inv))*lx1_ct/2, 0, 0]; % Coordinates of the centre point of the geometry
                        [M, N]=LensPlaneSurface_x(10, M, N, lx1_b_coord, lx1_dmax, lx1_n, 'Outlet', 0, 0);
                    end
                end
                
            else if lx1_Plano==2
                    
                    % Laser on the plane surface of the first lens (lx1) in front of the reflective lens (CS 9)
                    lx1_b_coord=[xg1-lx1_ct/2, 0, 0]; % Coordinates of the centre point of the geometry
                    [M, N]=LensPlaneSurface_x(9, M, N, lx1_b_coord, lx1_dmax, lx1_n, 'Inlet', 0, 0);
                    
                    
                    % Laser on the plane surface of the first lens (lx1) in front of the reflective lens (CS 10)
                    lx1_b_coord=[xg1+lx1_ct/2, 0, 0]; % Coordinates of the centre point of the geometry
                    [M, N]=LensPlaneSurface_x(10, M, N, lx1_b_coord, lx1_dmax, lx1_n, 'Outlet', 0, 0);
                end
            end
        end
        
    else % Only outlet on an integrated design borescope
        M(:,:,9)=M(:,:,8);
        
        if lx1_Plano==2 % Flat surface outlet
            
            % Laser on the plane surface of the first lens (lx1) in front of the reflective lens (CS 10)
            lx1_b_coord=[xg1+lx1_ct/2, 0, 0]; % Coordinates of the centre point of the geometry
            [M, N]=LensPlaneSurface_x(10, M, N, lx1_b_coord, lx1_dmax, lx1_n, 'Outlet', 0, 0);
            
        else
            % Laser on the curved surface of the first lens (lx1) in front of the reflective lens (CS 10)
            lx1_c_coord=[xg1+lx1_ct/2+((-1)^(lx1_cc))*lx1_R, 0, 0]; % Coordinates of the centre point of the geometry
            [M, N]=LensCurvedSurface_x(10, M, N, 0, lx1_R, lx1_c_coord, lx1_type, lx1_cc, lx1_dmax, lx1_n, 'Outlet', 0, 0);
            
        end
    end
    
    
else
    M(:,:,9)=M(:,:,8); % lens lx1 is ignored if number of x lens is smaller than 1
    M(:,:,10)=M(:,:,9); % lens lx1 is ignored if number of x lens is smaller than 1
end



%% lx2

if lx_N==2
    
    
    if Modular==1
        if lx2_Plano==0
            
            % Laser on the curved surface of the second lens (lx2) in front of the reflective lens (CS 11)
            lx2_c_coord=[xg2-lx2_ct/2-((-1)^(lx2_cc))*lx2_R, 0, 0]; % Coordinates of the centre point of the geometry
            [M, N]=LensCurvedSurface_x(11, M, N, 1, lx2_R, lx2_c_coord, lx2_type, lx2_cc, lx2_dmax, lx2_n, 'Inlet', 0, 0);
            
            % Laser on the curved surface of the second lens (lx2) in front of the reflective lens (CS 12)
            lx2_c_coord=[xg2+lx2_ct/2+((-1)^(lx2_cc))*lx2_R, 0, 0]; % Coordinates of the centre point of the geometry
            [M, N]=LensCurvedSurface_x(12, M, N, 0, lx2_R, lx2_c_coord, lx2_type, lx2_cc, lx2_dmax, lx2_n, 'Outlet', 0, 0);
            
            
        else if lx2_Plano==1
                
                if lx2_Inv==0
                    
                    % Laser on the plane surface of the second lens (lx2) in front of the reflective lens (CS 11)
                    lx2_b_coord=[xg2-((-1)^(lx2_Inv))*lx2_ct/2, 0, 0]; % Coordinates of the centre point of the geometry
                    [M, N]=LensPlaneSurface_x(11, M, N, lx2_b_coord, lx2_dmax, lx2_n, 'Inlet', 0, 0);
                    
                    % Laser on the curved surface of the second lens (lx2) in front of the reflective lens (CS 12)
                    lx2_c_coord=[xg2+((-1)^(lx2_Inv))*lx2_ct/2+((-1)^(lx2_cc+lx2_Inv))*lx2_R, 0, 0]; % Coordinates of the centre point of the geometry
                    [M, N]=LensCurvedSurface_x(12, M, N, lx2_Inv, lx2_R, lx2_c_coord, lx2_type, lx2_cc, lx2_dmax, lx2_n, 'Outlet', 0, 0);
                    
                else if lx2_Inv==1
                        
                        % Laser on the curved surface of the second lens (lx2) in front of the reflective lens (CS 11)
                        lx2_c_coord=[xg2+((-1)^(lx2_Inv))*lx2_ct/2+((-1)^(lx2_cc+lx2_Inv))*lx2_R, 0, 0]; % Coordinates of the centre point of the geometry
                        [M, N]=LensCurvedSurface_x(11, M, N, lx2_Inv, lx2_R, lx2_c_coord, lx2_type, lx2_cc, lx2_dmax, lx2_n, 'Inlet', 0, 0);
                        
                        % Laser on the plane surface of the second lens (lx2) in front of the reflective lens (CS 12)
                        lx2_b_coord=[xg2-((-1)^(lx2_Inv))*lx2_ct/2, 0, 0]; % Coordinates of the centre point of the geometry
                        [M, N]=LensPlaneSurface_x(12, M, N, lx2_b_coord, lx2_dmax, lx2_n, 'Outlet', 0, 0);
                        
                    end
                end
                
            else if lx2_Plano==2
                    % Laser on the plane surface of the second lens (lx2) in front of the reflective lens (CS 11)
                    lx2_b_coord=[xg2-lx2_ct/2, 0, 0]; % Coordinates of the centre point of the geometry
                    [M, N]=LensPlaneSurface_x(11, M, N, lx2_b_coord, lx2_dmax, lx2_n, 'Inlet', 0, 0);
                    
                    
                    % Laser on the plane surface of the second lens (lx2) in front of the reflective lens (CS 12)
                    lx2_b_coord=[xg2+lx2_ct/2, 0, 0]; % Coordinates of the centre point of the geometry
                    [M, N]=LensPlaneSurface_x(12, M, N, lx2_b_coord, lx2_dmax, lx2_n, 'Outlet', 0, 0);
                end
            end
        end
        
    else % Not applicable to  an integrated design borescope
        M(:,:,11)=M(:,:,10);
        M(:,:,12)=M(:,:,11);
    end
    
else
    M(:,:,11)=M(:,:,10); % lens lx2 is ignored if number of x lens is smaller than 2
    M(:,:,12)=M(:,:,11); % lens lx2 is ignored if number of x lens is smaller than 2
end

%% Laser on the inside of the Enclosure (CS 13)
if WithinEnclosure==1
    
    
    for k=1:1:length(M(:,1,1))
        
        
        if M(k,8,12)==1
            M(k,1:8,13)=[M(k,1:6,12) 3 M(k,8,12)]; % Laser ray failed before
        else
            
            
            % Laser ray vector
            a13=M(k,4,12)-M(k,1,12);
            b13=M(k,5,12)-M(k,2,12);
            c13=M(k,6,12)-M(k,3,12);
            
            % Plane Coefficients
            A13=Win_is(1,13);
            B13=Win_is(1,14);
            C13=Win_is(1,15);
            D13=Win_is(1,16);
            
            %%% Intersection of the laser ray with the geometry
            
            % Intersection equations
            M(k,1,13)=(B13*b13/a13*M(k,1,12)-B13*M(k,2,12)+C13*c13/a13*M(k,1,12)-C13*M(k,3,12)-D13)/(A13+B13*b13/a13+C13*c13/a13);
            M(k,2,13)=b13/a13*(M(k,1,13)-M(k,1,12))+M(k,2,12);
            M(k,3,13)=c13/a13*(M(k,1,13)-M(k,1,12))+M(k,3,12);
            
            N(k,1:3,13)=[A13 B13 C13];
            
            if (M(k,1,13)>=Win_is(1,17) && M(k,1,13)<=Win_is(1,18) && M(k,2,13)>=Win_is(1,19) && M(k,2,13)<=Win_is(1,20) && (((M(k,1,13)-M(k,1,12)>=0) && (M(k,4,12)-M(k,1,12)>=0)) || ((M(k,1,13)-M(k,1,12)<=0) && (M(k,4,12)-M(k,1,12)<=0))))
                [M(k,4:6,13), M(k,7,13), M(k,8,13)]=Optics(M(k,1:3,12), M(k,1:3,13), N(k,1:3,13), 1/lE_n, 0, 0);
                
            else % Laser ray out of lens
                M(k,1:6,13)=M(k,1:6,12);
                M(k,7:8,13)=[4 1];
                
            end
            
            
        end
        
        
    end
    
    
    %% Laser on the outside of the Enclosure (CS 14)
    
    for k=1:1:length(M(:,1,1))
        
        
        if M(k,8,13)==1
            M(k,1:8,14)=[M(k,1:6,13) 3 M(k,8,13)]; % Laser ray failed before
        else
            
            
            % Laser ray vector
            a14=M(k,4,13)-M(k,1,13);
            b14=M(k,5,13)-M(k,2,13);
            c14=M(k,6,13)-M(k,3,13);
            
            v=1;
            while ((M(k,1,14)+0.001<Win_os(v,17)) || (M(k,1,14)-0.001>Win_os(v,18)) || (M(k,2,14)+0.001<Win_os(v,19)) || (M(k,2,14)-0.001>Win_os(v,20)) || (M(k,2,14)<((Win_is_0(2,2)-Win_is_0(2,1))/(Win_is_0(1,2)-Win_is_0(1,1)))*M(k,1,14)+(Win_is_0(2,2)-((Win_is_0(2,2)-Win_is_0(2,1))/(Win_is_0(1,2)-Win_is_0(1,1)))*Win_is_0(1,2))))
                
                % Plane Coefficients
                A14=Win_os(v,13);
                B14=Win_os(v,14);
                C14=Win_os(v,15);
                D14=Win_os(v,16);
                
                %%% Intersection of the laser ray with the geometry
                % Intersection equations
                M(k,1,14)=(B14*b14/a14*M(k,1,13)-B14*M(k,2,13)+C14*c14/a14*M(k,1,13)-C14*M(k,3,13)-D14)/(A14+B14*b14/a14+C14*c14/a14);
                M(k,2,14)=b14/a14*(M(k,1,14)-M(k,1,13))+M(k,2,13);
                M(k,3,14)=c14/a14*(M(k,1,14)-M(k,1,13))+M(k,3,13);
                
                N(k,1:3,14)=[A14 B14 C14];
                
                
                v=v+1;
            end
            
            
            [M(k,4:6,14), M(k,7,14), M(k,8,14)]=Optics(M(k,1:3,13), M(k,1:3,14), N(k,1:3,14), lE_n, 0, 0);
            
            
        end
        
        
    end
    
else
    M(:,1:8,13)=M(:,1:8,12); % not applicable without enclosure
    M(:,1:8,14)=M(:,1:8,13); % not applicable without enclosure
    
end


%% Laser on delivery area (CS 15)

for k=1:1:length(M(:,1,1))
    
    
    if M(k,8,14)==1
        
        M(k,1:8,15)=[M(k,1:6,14) 3 M(k,8,14)]; % Laser ray failed before
    else
        
        
        % Laser ray vector
        a15=M(k,4,14)-M(k,1,14);
        b15=M(k,5,14)-M(k,2,14);
        c15=M(k,6,14)-M(k,3,14);
        
        % Coordinates of the centre point of the geometry
        l_Delivery=[Delivery, 0, 0];
        
        % Plane Coefficients
        A15=1;
        B15=0;
        C15=0;
        D15=-l_Delivery(1,1);
        
        %%% Intersection of the laser ray with the geometry
        % Intersection equations
        M(k,1,15)=(B15*b15/a15*M(k,1,14)-B15*M(k,2,14)+C15*c15/a15*M(k,1,14)-C15*M(k,3,14)-D15)/(A15+B15*b15/a15+C15*c15/a15);
        M(k,2,15)=b15/a15*(M(k,1,15)-M(k,1,14))+M(k,2,14);
        M(k,3,15)=c15/a15*(M(k,1,15)-M(k,1,14))+M(k,3,14);
        
    end
    
end

%% Rotation to reference frame of the Enclosure

for l=1:1:length(M(1,1,:))
    
    for k=1:1:length(M(:,1,1))
        M0(k,1,l)=cos(-rot)*(M(k,1,l))+sin(-rot)*(M(k,2,l));
        M0(k,2,l)=-sin(-rot)*(M(k,1,l))+cos(-rot)*(M(k,2,l));
        M0(k,3,l)=M(k,3,l);
    end
    
end


%% Power Output


if lz_N==2 % Lens that contributed to shaping the beam
    lz2_toggle=1;
    lz1_toggle=1;
end

if lz_N==1
    lz2_toggle=0;
    lz1_toggle=1;
end

if lz_N==0
    lz2_toggle=0;
    lz1_toggle=0;
end

if lx_N==2
    lz2_toggle=1;
    lz1_toggle=1;
end

if lx_N==1
    lx2_toggle=0;
    lx1_toggle=1;
end

if lx_N==0
    lx2_toggle=0;
    lx1_toggle=0;
end

I_beam=0; % Non-dimensional beam power
I_inlet=0; % Non-dimensional inlet power
I_inlet_max=0; % Non-dimensional inlet power (core only)
I_outlet=0; % Non-dimensional outlet power
I_outlet_max=0; % Non-dimensional outlet power (core only)
dr_b=Rc(2)-Rc(1);
dr_a=a(2)-a(1);
dtheta=thetac(2)-thetac(1);
Trss=(lz2_trss^lz2_toggle)*(lz1_trss^lz1_toggle)*lR_trss*(lx1_trss^lx1_toggle)*(lx2_trss^lx2_toggle)*(lE_trss^WithinEnclosure); % Laser power transmited

% Sector area evaluation for each mesh point
for k=2:1:length(M(:,1,1))
    
    
    Sector=[Ellipses(k,1)-dr_a/2 Ellipses(k,2)-dr_b/2 Ellipses(k,3)+dtheta/2;
        Ellipses(k,1)-dr_a/2 Ellipses(k,2)-dr_b/2 Ellipses(k,3)-dtheta/2;
        Ellipses(k,1)+dr_a/2 Ellipses(k,2)+dr_b/2 Ellipses(k,3)+dtheta/2;
        Ellipses(k,1)+dr_a/2 Ellipses(k,2)+dr_b/2 Ellipses(k,3)-dtheta/2];
    
    for pps=1:1:4
        if (((0<=Sector(pps,3)) && (Sector(pps,3)<=pi/2)) || ((3*pi/2<Sector(pps,3)) && (Sector(pps,3)<=2*pi)))
            SectorPoints(pps,1:2)=[xd+sqrt(1/((1/Sector(pps,1)/Sector(pps,1))+(tan(Sector(pps,3))*tan(Sector(pps,3))/Sector(pps,2)/Sector(pps,2)))) ...
                yd+(sqrt(1/((1/Sector(pps,1)/Sector(pps,1))+(tan(Sector(pps,3))*tan(Sector(pps,3))/Sector(pps,2)/Sector(pps,2)))))*tan(Sector(pps,3))];
        else
            SectorPoints(pps,1:2)=[xd-sqrt(1/((1/Sector(pps,1)/Sector(pps,1))+(tan(Sector(pps,3))*tan(Sector(pps,3))/Sector(pps,2)/Sector(pps,2)))) ...
                yd-(sqrt(1/((1/Sector(pps,1)/Sector(pps,1))+(tan(Sector(pps,3))*tan(Sector(pps,3))/Sector(pps,2)/Sector(pps,2)))))*tan(Sector(pps,3))];
        end
    end
    
    Asector=polyarea(SectorPoints(:,1),SectorPoints(:,2));
    
    I_beam=Asector*Colour_Gaussian(k,1)+I_beam; % Total non-dimensional power beam
    
    
    if M(k,7,1)~=5 % Total non-dimensional power inlet
        I_inlet=Asector*Colour_Gaussian(k,1)+I_inlet;
        if Colour_Gaussian(k,1)==1
            I_inlet_max=Asector*Colour_Gaussian(k,1)+I_inlet_max;
            
        end
    end
    
    if sum(M(k,8,:))==0  % Total non-dimensional power outlet
        I_outlet=Asector*Colour_Gaussian(k,1)+I_outlet;
        if Colour_Gaussian(k,1)==1
            I_outlet_max=Asector*Colour_Gaussian(k,1)+I_outlet_max;
        end
    end
    
end

PowerOut=I_outlet/I_beam*PowerBeam*Trss; % Total power out

BoundaryIn=zeros(Np,2);
BoundaryInMax=zeros(Np,2);

s=1; % Determination of areas of core and beam at the inlet
for q=1:1:Np
    for r=1:Np:Nc*Np
        if M(q+r,7,1)~=5
            BoundaryIn(s,:)=M(q+r,1:2,1);
            if Colour_Gaussian(q+r,1)==1
                BoundaryInMax(s,:)=M(q+r,1:2,1);
            end
        end
    end
    s=s+1;
end
AreaIn=polyarea(BoundaryIn(:,1),BoundaryIn(:,2)); % Inlet area of the beam
AreaInMax=polyarea(BoundaryInMax(:,1),BoundaryInMax(:,2));  % Inlet area of the core

BoundaryOut=zeros(Np,2);
BoundaryOutMax=zeros(Np,2);

s=1; % Determination of areas of core and beam at the outlet
for q=1:1:Np
    for r=1:Np:Nc*Np
        if sum(M(q+r,8,:))==0
            BoundaryOut(s,:)=M(q+r,2:3,15);
            if Colour_Gaussian(q+r,1)==1
                BoundaryOutMax(s,:)=M(q+r,2:3,15);
            end
            
        end
        
    end
    s=s+1;
end
AreaOut=polyarea(BoundaryOut(:,1),BoundaryOut(:,2));  % Outlet area of the beam
AreaOutMax=polyarea(BoundaryOutMax(:,1),BoundaryOutMax(:,2));  % Outlet area of the core

PDAIn=PowerBeam*I_inlet/I_beam/AreaIn/1000*100; % J/cm2 Power Density Average Inlet
PDMIn=I_inlet_max/I_beam*PowerBeam/AreaInMax/1000*100; % J/cm2 Power Density Mximum Inlet
PDAOut=PowerOut/AreaOut/1000*100; % J/cm2 Power Density Average Outlet
PDMOut=I_outlet_max/I_outlet*PowerOut/AreaOutMax/1000*100; % J/cm2 Power Density Mlet



%% Plot

WM=msgbox('Please Wait...');

if Plot==1
    
    
    figure;
    subplot(2,2,1)
    for i=1:1:length(M(:,1,1))
        if sum(M(i,7,1))==5
            scatter(M(i,2,1), M(i,1,1), 7, 'k');
        else
            scatter(M(i,2,1), M(i,1,1), 7, Colour_Gaussian(i,2:4));
        end
        hold on;
    end
    for i=1:1:length(BoundaryIn(:,1))
        scatter(BoundaryIn(i,2), BoundaryIn(i,1), 40, 'k');
        hold on;
    end
    for i=1:1:length(BoundaryInMax(:,1))
        scatter(BoundaryInMax(i,2), BoundaryInMax(i,1), 40, 'c');
        hold on;
    end
    axis equal;
    ax1 = gca; % current axes
    set(ax1,'XTick',-D/2:D/5:D/2);
    axis(ax1, [-D/2 D/2 -ER*D/2 ER*D/2 ]);
    set(ax1,'YTick',-ER*D/2:ER*D/5:ER*D/2, 'FontSize', 12);
    set(ax1,'YDir','reverse')
    ylabel(ax1,'x [mm]', 'FontSize', 14, 'FontWeight','bold');
    xlabel(ax1,'y [mm]', 'FontSize', 14, 'FontWeight','bold');
    title(sprintf('Inlet Beam, Diameter = %.2f mm \n Power Beam= %.2fmJ, Power In= %.2fmJ \n PDA = %.2fJ/cm^2, PDM = %.2fJ/cm^2', D, PowerBeam, PowerBeam*I_inlet/I_beam, PDAIn, PDMIn));
    grid on;
    hold on;
    
    subplot(2,2,3)
    
    
    
    
    for i=1:1:length(M(:,1,1))
        if sum(M(i,8,:))>0
            scatter(M(i,2,1), M(i,1,1), 7, 'k');
        else
            scatter(M(i,2,1), M(i,1,1), 7, Colour_Gaussian(i,2:4));
        end
        
        
        hold on;
    end
    axis equal;
    ax1 = gca; % current axes
    axis(ax1, [-D/2 D/2 -ER*D/2 ER*D/2]);
    ylabel(ax1,'x [mm]', 'FontSize', 14, 'FontWeight','bold');
    xlabel(ax1,'y [mm]', 'FontSize', 14, 'FontWeight','bold');
    set(ax1,'YTick',-ER*D/2:ER*D/5:ER*D/2, 'FontSize', 12);
    set(ax1,'XTick',-D/2:D/5:D/2);
    
    set(ax1,'YDir','reverse')
    title(sprintf('Useful Portion of the Inlet Beam, Power = %.2fmJ', PowerOut));
    grid on
    hold on;
    subplot(2,2,2)
    
    if WithinEnclosure==1
        for i=1:1:length(M(:,1,1))
            
            if sum(M(i,8,:))>0
                colorplotline='k';
            else
                colorplotline=Colour_Gaussian(i,2:4);
            end
            
            
            plot3([M0(i,1,1) M0(i,1,2) M0(i,1,3) M0(i,1,4) M0(i,1,5) M0(i,1,6) M0(i,1,7) M0(i,1,8) M0(i,1,9) M0(i,1,10) M0(i,1,11) M0(i,1,12) M0(i,1,13) M0(i,1,14) M0(i,1,15)],...
                [M0(i,2,1) M0(i,2,2) M0(i,2,3) M0(i,2,4) M0(i,2,5) M0(i,2,6) M0(i,2,7) M0(i,2,8) M0(i,2,9) M0(i,2,10) M0(i,2,11) M0(i,2,12) M0(i,2,13) M0(i,2,14) M0(i,2,15)],...
                [M0(i,3,1) M0(i,3,2) M0(i,3,3) M0(i,3,4) M0(i,3,5) M0(i,3,6) M0(i,3,7) M0(i,3,8) M0(i,3,9) M0(i,3,10) M0(i,3,11) M0(i,3,12) M0(i,3,13) M0(i,3,14) M0(i,3,15)], 'Color' , colorplotline);
            
            hold on;
        end
        plot(X_enclosure_0, Y_enclosure_0);
        
        plot(Win_is_0(1,1:2),Win_is_0(2,1:2));
        axis equal;
        
    else
        
        for i=1:1:length(M(:,1,1))
            
            if sum(M(i,8,:))>0
                colorplotline='k';
            else
                colorplotline=Colour_Gaussian(i,2:4);
            end
            
            % Plot of each laser ray at all the control surfaces
            plot3([M(i,1,1) M(i,4,1) M(i,1,2) M(i,1,3) M(i,1,4) M(i,1,5) M(i,1,6) M(i,1,7) M(i,1,8) M(i,1,9) M(i,1,10) M(i,1,11) M(i,1,12) M(i,1,15)],...
                [M(i,2,1) M(i,5,1) M(i,2,2) M(i,2,3) M(i,2,4) M(i,2,5) M(i,2,6) M(i,2,7) M(i,2,8) M(i,2,9) M(i,2,10) M(i,2,11) M(i,2,12) M(i,2,15)],...
                [M(i,3,1) M(i,6,1)  M(i,3,2) M(i,3,3) M(i,3,4) M(i,3,5) M(i,3,6) M(i,3,7) M(i,3,8) M(i,3,9) M(i,3,10) M(i,3,11) M(i,3,12) M(i,3,15)], 'Color' , colorplotline);
            hold on;
        end
        axis equal;
        
    end
    
    title('Beam pathway');
    ax1 = gca; % current axes
    set(ax1,'FontSize', 12);
    xlabel(ax1,'x [mm]', 'FontSize', 14, 'FontWeight','bold');
    ylabel(ax1,'y [mm]', 'FontSize', 14, 'FontWeight','bold');
    zlabel(ax1,'z [mm]', 'FontSize', 14, 'FontWeight','bold');
    hold on;
    subplot(2,2,4)
    for i=1:1:length(M(:,1,1))
        if sum(M(i,8,:))==0
            scatter(M(i,2,15), M(i,3,15), 7, Colour_Gaussian(i,2:4));
        end
        hold on;
    end
    for i=1:1:length(BoundaryOut(:,1))
        scatter(BoundaryOut(i,1), BoundaryOut(i,2), 40, 'k');
        hold on;
    end
    for i=1:1:length(BoundaryOutMax(:,1))
        scatter(BoundaryOutMax(i,1), BoundaryOutMax(i,2), 40, 'c');
        hold on;
    end
    axis equal;
    ax1 = gca; % current axes
    set(ax1,'FontSize', 12);
    xlabel(ax1,'y [mm]', 'FontSize', 14, 'FontWeight','bold');
    ylabel(ax1,'z [mm]', 'FontSize', 14, 'FontWeight','bold');
    title(sprintf('Beam cross-section at a distance = %.2fmm \n PDA = %.2fJ/cm^2, PDM = %.2fJ/cm^2', Delivery, PDAOut, PDMOut));
    grid on;
    
    fig = gcf;
    set(gcf, 'Position', get(0, 'Screensize'));
    print(gcf,strcat(OutputDirectory,'\Borescope.tiff'),'-dtiffn',sprintf('-r%.0f', PlotResolution));
    print(gcf,strcat(OutputDirectory,'\Borescope.png'),'-dpng',sprintf('-r%.0f', PlotResolution));
    savefig(strcat(OutputDirectory,'\Borescope.fig'));
    fig.PaperPositionMode = 'manual';
    orient(fig,'landscape');
    print(fig,strcat(OutputDirectory,'\Borescope.pdf'),'-dpdf',sprintf('-r%.0f', PlotResolution));
    
    
    if PlotIndividual==1
        
        
        figure;
        for i=1:1:length(M(:,1,1))
            if sum(M(i,7,1))==5
                scatter(M(i,2,1), M(i,1,1), 7, 'k');
            else
                scatter(M(i,2,1), M(i,1,1), 7, Colour_Gaussian(i,2:4));
            end
            hold on;
        end
        for i=1:1:length(BoundaryIn(:,1))
            scatter(BoundaryIn(i,2), BoundaryIn(i,1), 40, 'k');
            hold on;
        end
        for i=1:1:length(BoundaryInMax(:,1))
            scatter(BoundaryInMax(i,2), BoundaryInMax(i,1), 40, 'c');
            hold on;
        end
        axis equal;
        ax1 = gca; % current axes
        set(ax1,'XTick',-D/2:D/5:D/2);
        ylabel(ax1,'x [mm]', 'FontSize', 14, 'FontWeight','bold');
        xlabel(ax1,'y [mm]', 'FontSize', 14, 'FontWeight','bold');
        title('Inlet Beam');
        grid on;
        axis(ax1, [-D/2 D/2 -ER*D/2 ER*D/2 ]);
        set(ax1,'YTick',-ER*D/2:ER*D/5:ER*D/2, 'FontSize', 12);
        set(ax1,'YDir','reverse')
        ax1.FontName='arial';
        ax2.FontName='arial';
        set(gcf, 'PaperPositionMode', 'auto')
        print(gcf,strcat(OutputDirectory,'\BorescopeInlet.tiff'),'-dtiffn',sprintf('-r%.0f', PlotResolution));
        print(gcf,strcat(OutputDirectory,'\BorescopeInlet.png'),'-dpng',sprintf('-r%.0f', PlotResolution));
        savefig(strcat(OutputDirectory,'\BorescopeInlet.fig'));
        fig.PaperPositionMode = 'manual';
        orient(fig,'landscape');
        print(gcf,strcat(OutputDirectory,'\BorescopeInlet.pdf'),'-dpdf',sprintf('-r%.0f', PlotResolution));
        
        
        figure;
        for i=1:1:length(M(:,1,1))
            if sum(M(i,8,:))>0
                scatter(M(i,2,1), M(i,1,1), 7, 'k');
            else
                scatter(M(i,2,1), M(i,1,1), 7, Colour_Gaussian(i,2:4));
            end
            
            
            hold on;
        end
        axis equal;
        ax1 = gca; % current axes
        axis(ax1, [-D/2 D/2 -ER*D/2 ER*D/2]);
        ylabel(ax1,'x [mm]', 'FontSize', 14, 'FontWeight','bold');
        xlabel(ax1,'y [mm]', 'FontSize', 14, 'FontWeight','bold');
        set(ax1,'YTick',-ER*D/2:ER*D/5:ER*D/2, 'FontSize', 12);
        set(ax1,'XTick',-D/2:D/5:D/2);
        
        set(ax1,'YDir','reverse')
        set(fig, 'PaperPositionMode', 'auto')
        title('Useful Portion of the Inlet Beam');
        grid on
        print(gcf,strcat(OutputDirectory,'\BorescopeUsefulInlet.tiff'),'-dtiffn',sprintf('-r%.0f', PlotResolution));
        print(gcf,strcat(OutputDirectory,'\BorescopeUsefulInlet.png'),'-dpng',sprintf('-r%.0f', PlotResolution));
        savefig(strcat(OutputDirectory,'\BorescopeUsefulInlet.fig'));
        fig.PaperPositionMode = 'manual';
        orient(fig,'landscape');
        print(gcf,strcat(OutputDirectory,'\BorescopeUsefulInlet.pdf'),'-dpdf',sprintf('-r%.0f', PlotResolution));
        
        
        figure;
        for i=1:1:length(M(:,1,1))
            if sum(M(i,8,:))==0
                scatter(M(i,2,15), M(i,3,15), 7, Colour_Gaussian(i,2:4));
            end
            hold on;
        end
        for i=1:1:length(BoundaryOut(:,1))
            scatter(BoundaryOut(i,1), BoundaryOut(i,2), 40, 'k');
            hold on;
        end
        for i=1:1:length(BoundaryOutMax(:,1))
            scatter(BoundaryOutMax(i,1), BoundaryOutMax(i,2), 40, 'c');
            hold on;
        end
        axis equal;
        ax1 = gca; % current axes
        set(ax1,'FontSize', 12);
        xlabel(ax1,'y [mm]', 'FontSize', 14, 'FontWeight','bold');
        ylabel(ax1,'z [mm]', 'FontSize', 14, 'FontWeight','bold');
        title(sprintf('Beam cross-section at a distance = %.2fmm', Delivery));
        grid on;
        print(gcf,strcat(OutputDirectory,'\BorescopeCrossSectionOutlet.tiff'),'-dtiffn',sprintf('-r%.0f', PlotResolution));
        print(gcf,strcat(OutputDirectory,'\BorescopeCrossSectionOutlet.png'),'-dpng',sprintf('-r%.0f', PlotResolution));
        savefig(strcat(OutputDirectory,'\BorescopeCrossSectionOutlet.fig'));
        fig.PaperPositionMode = 'manual';
        orient(fig,'landscape');
        print(gcf,strcat(OutputDirectory,'\BorescopeCrossSectionOutlet.pdf'),'-dpdf',sprintf('-r%.0f', PlotResolution));
        
        
        figure;
        if WithinEnclosure==1
            for i=1:1:length(M(:,1,1))
                if sum(M(i,8,:))>0
                    colorplotline='k';
                else
                    colorplotline=Colour_Gaussian(i,2:4);
                end
                
                
                plot3([M0(i,1,1) M0(i,1,2) M0(i,1,3) M0(i,1,4) M0(i,1,5) M0(i,1,6) M0(i,1,7) M0(i,1,8) M0(i,1,9) M0(i,1,10) M0(i,1,11) M0(i,1,12) M0(i,1,13) M0(i,1,14) M0(i,1,15)],...
                    [M0(i,2,1) M0(i,2,2) M0(i,2,3) M0(i,2,4) M0(i,2,5) M0(i,2,6) M0(i,2,7) M0(i,2,8) M0(i,2,9) M0(i,2,10) M0(i,2,11) M0(i,2,12) M0(i,2,13) M0(i,2,14) M0(i,2,15)],...
                    [M0(i,3,1) M0(i,3,2) M0(i,3,3) M0(i,3,4) M0(i,3,5) M0(i,3,6) M0(i,3,7) M0(i,3,8) M0(i,3,9) M0(i,3,10) M0(i,3,11) M0(i,3,12) M0(i,3,13) M0(i,3,14) M0(i,3,15)], 'Color' , colorplotline);
                
                hold on;
            end
            plot(X_enclosure_0, Y_enclosure_0);
            
            plot(Win_is_0(1,1:2),Win_is_0(2,1:2));
            axis equal;
            
        else
            
            for i=1:1:length(M(:,1,1))
                
                if sum(M(i,8,:))>0
                    colorplotline='k';
                else
                    colorplotline=Colour_Gaussian(i,2:4);
                end
                
                % Plot of each laser ray at all the control surfaces
                plot3([M(i,1,1) M(i,4,1) M(i,1,2) M(i,1,3) M(i,1,4) M(i,1,5) M(i,1,6) M(i,1,7) M(i,1,8) M(i,1,9) M(i,1,10) M(i,1,11) M(i,1,12) M(i,1,15)],...
                    [M(i,2,1) M(i,5,1) M(i,2,2) M(i,2,3) M(i,2,4) M(i,2,5) M(i,2,6) M(i,2,7) M(i,2,8) M(i,2,9) M(i,2,10) M(i,2,11) M(i,2,12) M(i,2,15)],...
                    [M(i,3,1) M(i,6,1)  M(i,3,2) M(i,3,3) M(i,3,4) M(i,3,5) M(i,3,6) M(i,3,7) M(i,3,8) M(i,3,9) M(i,3,10) M(i,3,11) M(i,3,12) M(i,3,15)], 'Color' , colorplotline);
                hold on;
            end
            axis equal;
            
        end
        title('Beam pathway');
        ax1 = gca; % current axes
        set(ax1,'FontSize', 12);
        xlabel(ax1,'x [mm]', 'FontSize', 14, 'FontWeight','bold');
        ylabel(ax1,'y [mm]', 'FontSize', 14, 'FontWeight','bold');
        zlabel(ax1,'z [mm]', 'FontSize', 14, 'FontWeight','bold');
        ax1.FontName='arial';
        
        set(fig, 'PaperPositionMode', 'auto')
        print(gcf,strcat(OutputDirectory,'\BorescopeBeamPathway.tiff'),'-dtiffn',sprintf('-r%.0f', PlotResolution));
        print(gcf,strcat(OutputDirectory,'\BorescopeBeamPathway.png'),'-dpng',sprintf('-r%.0f', PlotResolution));
        savefig(strcat(OutputDirectory,'\BorescopeBeamPathway.fig'));
        fig.PaperPositionMode = 'manual';
        orient(fig,'landscape');
        print(fig,strcat(OutputDirectory,'\BorescopeBeamPathway.pdf'),'-dpdf',sprintf('-r%.0f', PlotResolution));
        
    end
end

fid = fopen(strcat(OutputDirectory,'\MatrixResults.txt'),'wt');
fprintf(fid,'MATRIX RESULTS\n');
fprintf(fid,'Columns are x, y, z, xd, yd, zd, OB, STOP\n\n');
for CS=1:1:15
    fprintf(fid,sprintf('CS%i\n',CS));
    fprintf(fid,'%.5f %.5f %.5f %.5f %.5f %.5f %.5f %.5f\n',M(:,:,CS)');
    fprintf(fid,'\n\n');
end
fclose(fid);

close(WM);
msgbox(strcat('Operation Completed. Results in folder', {' '}, OutputDirectory, '.'));

