function [M, N]=LensCurvedSurface_x(CS, M, N, lx_Inv, lx_R, lx_c_coord, lx_type, lx_cc, lx_dmax, n, Type, FR, EX)
% M: Matrix with the control points
% N: Matrix with the plane normal at the control points
% CS: control surface
% lx_Inv: if this lx lens is inverted or not
% lx_R: radius of the lens
% lx_c_coord: coordinates of the geometrical centre of the lens
% lx_type: type of lens: 0 for spread, 1 for thickness, 2 for both
% lx_cc: centre thickness of the lens
% lx_dmax: diameter of the maximum useful area of the lens
% n: ratio of refractive index of the optical medium to the gas medium
% Type: type of control surface: 'Inlet' or 'Outlet'
% FR: forced reflection (only applicable for surfaces with reflective coatings)
% EX: expected behaviour at the current control surface: 0 for refraction, 1 for reflection

if strcmp(Type,'Outlet')==1
    nr=n;
else if strcmp(Type,'Inlet')==1
        nr=1/n;
    end
end

for k=1:1:length(M(:,1,1))
    
    if M(k,8,CS-1)==1
        M(k,1:8,CS)=[M(k,1:6,CS-1) 3 M(k,8,CS-1)]; % Laser ray failed before
        
    else
        % Laser ray vector
        a=M(k,4,CS-1)-M(k,1,CS-1);
        b=M(k,5,CS-1)-M(k,2,CS-1);
        c=M(k,6,CS-1)-M(k,3,CS-1);
        
        %%% Intersection of the laser ray with the geometry (depends on the
        %%% geometry type)
        if lx_type==0
            
            % Quadratic formula a,b,c coefficients
            ai=(b/a)^2+1;
            bi=2*b/a*(M(k,2,CS-1)-lx_c_coord(1,2))-2*lx_c_coord(1,1)-2*M(k,1,CS-1)*(b/a)^2;
            ci=M(k,1,CS-1)^2*(b/a)^2+(M(k,2,CS-1)-lx_c_coord(1,2))^2+lx_c_coord(1,1)^2-lx_R^2-2*b/a*(M(k,2,CS-1)-lx_c_coord(1,2))*M(k,1,CS-1);
            
            % Intersection equations
            M(k,1,CS)=(-bi+((-1)^(lx_cc+lx_Inv+1))*sqrt(bi^2-4*ai*ci))/(2*ai);
            M(k,2,CS)=b/a*(M(k,1,CS)-M(k,1,CS-1))+M(k,2,CS-1);
            M(k,3,CS)=c/a*(M(k,1,CS)-M(k,1,CS-1))+M(k,3,CS-1);
            
            N(k,1:3,CS)=[M(k,1,CS)-lx_c_coord(1,1) M(k,2,CS)-lx_c_coord(1,2) 0];
            
        else if lx_type==1
                
                % Quadratic formula a,b,c coefficients
                ai=(c/a)^2+1;
                bi=2*c/a*(M(k,3,CS-1)-lx_c_coord(1,3))-2*lx_c_coord(1,1)-2*M(k,1,CS-1)*(c/a)^2;
                ci=M(k,1,CS-1)^2*(c/a)^2+(M(k,3,CS-1)-lx_c_coord(1,3))^2+lx_c_coord(1,1)^2-lx_R^2-2*c/a*(M(k,3,CS-1)-lx_c_coord(1,3))*M(k,1,CS-1);
                
                % Intersection equations
                M(k,1,CS)=(-bi+((-1)^(lx_cc+lx_Inv+1))*sqrt(bi^2-4*ai*ci))/(2*ai);
                M(k,2,CS)=b/a*(M(k,1,CS)-M(k,1,CS-1))+M(k,2,CS-1);
                M(k,3,CS)=c/a*(M(k,1,CS)-M(k,1,CS-1))+M(k,3,CS-1);
                
                N(k,1:3,CS)=[M(k,1,CS)-lx_c_coord(1,1) 0 M(k,3,CS)-lx_c_coord(1,3)];
                
            else if lx_type==2
                    
                    % Quadratic formula a,b,c coefficients
                    ai=(b/a)^2+(c/a)^2+1;
                    bi=-2*lx_c_coord(1,1)-2*M(k,1,CS-1)*(b/a)^2+2*b/a*(M(k,2,CS-1)-lx_c_coord(1,2))+...
                        -2*M(k,1,CS-1)*(c/a)^2+2*c/a*(M(k,3,CS-1)-lx_c_coord(1,3));
                    ci=M(k,1,CS-1)^2*(b/a)^2+(M(k,2,CS-1)-lx_c_coord(1,2))^2+lx_c_coord(1,1)^2-lx_R^2-2*b/a*(M(k,2,CS-1)-lx_c_coord(1,2))*M(k,1,CS-1)+...
                        M(k,1,CS-1)^2*(c/a)^2+(M(k,3,CS-1)-lx_c_coord(1,3))^2-2*c/a*(M(k,3,CS-1)-lx_c_coord(1,3))*M(k,1,CS-1);
                    
                    % Intersection equations
                    M(k,1,CS)=(-bi+((-1)^(lx_cc+lx_Inv+1))*sqrt(bi^2-4*ai*ci))/(2*ai);
                    M(k,2,CS)=b/a*(M(k,1,CS)-M(k,1,CS-1))+M(k,2,CS-1);
                    M(k,3,CS)=c/a*(M(k,1,CS)-M(k,1,CS-1))+M(k,3,CS-1);
                    
                    N(k,1:3,CS)=[lx_c_coord(1,1)-M(k,1,CS) lx_c_coord(1,2)-M(k,2,CS) lx_c_coord(1,3)-M(k,3,CS)];
                    
                end
            end
        end
        
        if sqrt(M(k,2,CS)^2+M(k,3,CS)^2)<=lx_dmax/2
            [M(k,4:6,CS), M(k,7,CS), M(k,8,CS)]=Optics(M(k,1:3,CS-1), M(k,1:3,CS), N(k,1:3,CS), nr, FR, EX);
            
        else % Laser ray out of lens
            
            M(k,7:8,CS)=[4 1];
            
        end
        
    end
    
end
end