function [M, N]=LensPlaneSurface_z(CS, M, N, lz_b_coord, lz_dmax, n, Type, FR, EX)
% M: Matrix with the control points
% N: Matrix with the plane normal at the control points
% CS: control surface
% lz_b_coord: coordinates of the flat surface of the lens
% lz_dmax: diameter of the maximum useful area of the lens
% n: ratio of refractive index of the optical medium to the gas medium
% Type: type of control surface: 'Inlet' or 'Outlet'
% FR: forced reflection (only applicable for surfaces with reflective coatings)
% EX: expected behaviour at the current control surface: 0 for refraction, 1 for reflection

if strcmp(Type,'Outlet')==1
    nr=n;
else if strcmp(Type,'Inlet')==1
        nr=1/n;
    end
end

for k=1:1:length(M(:,1,1))
    
    if M(k,8,CS-1)==1
        M(k,1:8,CS)=[M(k,1:6,CS-1) 3 M(k,8,CS-1)]; % Laser ray failed before
        
    else
        
        % Laser ray vector
        a=M(k,4,CS-1)-M(k,1,CS-1);
        b=M(k,5,CS-1)-M(k,2,CS-1);
        c=M(k,6,CS-1)-M(k,3,CS-1);
        
        % Plane Coefficients
        A=0;
        B=0;
        C=1;
        D=-lz_b_coord(1,3);
        
        %%% Intersection of the laser ray with the geometry
        % Intersection equations
        M(k,3,CS)=(A*a/c*M(k,3,CS-1)-A*M(k,1,CS-1)+B*b/c*M(k,3,CS-1)-B*M(k,2,CS-1)-D)/(A*a/c+B*b/c+C);
        M(k,2,CS)=b/c*(M(k,3,CS)-M(k,3,CS-1))+M(k,2,CS-1);
        M(k,1,CS)=a/c*(M(k,3,CS)-M(k,3,CS-1))+M(k,1,CS-1);
        
        N(k,1:3,CS)=[A B C];
         
        if sqrt(M(k,1,CS)^2+M(k,2,CS)^2)<=lz_dmax/2
            
            [M(k,4:6,CS), M(k,7,CS), M(k,8,CS)]=Optics(M(k,1:3,CS-1), M(k,1:3,CS), N(k,1:3,CS), nr, FR, EX);
            
        else % Laser ray out of lens
            
            M(k,7:8,CS)=[4 1];
            
        end
        
    end
    
end
end